@@ -21,7 +21,7 @@
  *   Chema Celorio <chema@ximian.com>
  *   Lauris Kaplinski <lauris@ximian.com>
  *
- * Copyright (C) 2001-2002 Ximian, Inc. 
+ * Copyright (C) 2001-2003 Ximian, Inc. 
  *
  */
 
@@ -33,6 +33,11 @@
 #include "gpa-widget.h"
 #include "gpa-option-menu.h"
 #include "gnome-print-i18n.h"
+#include "libgnomeprint/private/gnome-print-config-private.h"
+#include "libgnomeprint/private/gpa-node.h"
+#include "libgnomeprint/private/gpa-node-private.h"
+#include "libgnomeprint/private/gpa-key.h"
+#include "libgnomeprint/private/gpa-option.h"
 
 static void     gpa_option_menu_class_init (GPAOptionMenuClass *klass);
 static void     gpa_option_menu_init (GPAOptionMenu *om);
@@ -66,12 +71,10 @@ gpa_option_menu_class_init (GPAOptionMenuClass *klass)
 	GObjectClass *object_class;
 	GtkWidgetClass *gtk_class;
 
-	object_class = (GObjectClass *) klass;
-	gtk_class = (GtkWidgetClass *) klass;
-	
-	parent_class = gtk_type_class (GTK_TYPE_WIDGET);
-
-	object_class->finalize = gpa_option_menu_finalize;
+	object_class = (GObjectClass *)   klass;
+	gtk_class    = (GtkWidgetClass *) klass;
+	parent_class = gtk_type_class (GPA_TYPE_WIDGET);
+	object_class->finalize       = gpa_option_menu_finalize;
 	gtk_class->mnemonic_activate = gpa_option_menu_mnemonic_activate;
 }
 
@@ -81,14 +84,20 @@ gpa_option_menu_init (GPAOptionMenu *om)
 	om->menu = gtk_option_menu_new ();
 	gtk_container_add (GTK_CONTAINER (om), om->menu);
 	gtk_widget_show (GTK_WIDGET (om->menu));
-	
-	om->key    = NULL;
-	om->shell  = NULL;
-	om->updating = FALSE;
-	om->node   = NULL;
-	om->handler = 0;
-	om->parent = NULL;
-	om->parent_handler = 0;
+}
+
+static void
+gpa_option_menu_disconnect (GPAOptionMenu *om)
+{
+	if (om->handler) {
+		g_signal_handler_disconnect (om->node, om->handler);
+		om->handler = 0;
+	}
+
+	if (om->node) {
+		gpa_node_unref (om->node);
+		om->node = NULL;
+	}
 }
 
 static void
@@ -96,25 +105,18 @@ gpa_option_menu_finalize (GObject *object)
 {
 	GPAOptionMenu *om;
 
-	om = (GPAOptionMenu *) object;
-	
-	if (om->key)
-		g_free (om->key);
+	om = GPA_OPTION_MENU (object);
 
-	if (om->handler) {
-		g_signal_handler_disconnect (om->node, om->handler);
-		gpa_node_unref (om->node);
-		om->handler = 0;
-		om->node = NULL;
-	}
+	gpa_option_menu_disconnect (om);
+
+	if (om->handler_config)
+		g_signal_handler_disconnect (om->config, om->handler_config);
+	om->handler_config = 0;
+	om->config = NULL;
+
+	g_free (om->key);
+	om->key = NULL;
 
-	if (om->parent_handler) {
-		g_signal_handler_disconnect (om->parent, om->parent_handler);
-		gpa_node_unref (om->parent);
-		om->parent_handler = 0;
-		om->parent = NULL;
-	}
-	
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -123,186 +125,163 @@ gpa_option_menu_mnemonic_activate (GtkWidget *widget,
 				   gboolean group_cycling)
 {
 	gtk_widget_grab_focus (GPA_OPTION_MENU (widget)->menu);
-	
 	return TRUE;
 }
 
-
 static void
 gpa_option_menu_item_activate_cb (GtkMenuItem *item, GPAOptionMenu *om)
 {
-	GPANode *parent;
 	GPANode *node;
 
-	if (om->updating) {
-		g_print ("Avoid updating item_activated_cb\n");
-		return;
-	}
-
 	om->updating = TRUE;
-	
-	parent = gpa_node_get_child_from_path (GNOME_PRINT_CONFIG_NODE (GPA_WIDGET (om)->config), om->key);
 	node = g_object_get_data (G_OBJECT (item), "node");
-	gpa_node_set_value (parent, gpa_node_id (node));
-
+	gpa_node_set_value (om->node, gpa_node_id (node));
 	om->updating = FALSE;
 }
 
-static void
-gpa_option_menu_node_modified_cb (GPANode *node, guint flags, GPAOptionMenu *om)
-{
-	gchar *current;
-	GPANode *option, *item;
-	gint sel = 0, pos;
-
-	if (om->updating)
-		return;
-
-	current = gpa_node_get_value (node);
-	option = gpa_node_get_child_from_path (node, "Option");
-	g_assert (option);
-	g_assert (current);
-
-	pos = 0;
-	for (item = gpa_node_get_child (option, NULL); item != NULL; item = gpa_node_get_child (option, item)) {
-		gchar *id;
-		id = gpa_node_get_value (item);
-		if (*id && strcmp (id, current) == 0) {
-			sel = pos;
-			g_free (id);
-			break;
-		}
-		if (id)
-			g_free (id);
-		pos++;
-		gpa_node_unref (item);
-	}
-
-	om->updating = TRUE;
-	gtk_option_menu_set_history (GTK_OPTION_MENU (om->menu), sel);
-	om->updating = FALSE;
-
-	gpa_node_unref (option);
-	g_free (current);
-}
-
 static void
 gpa_option_menu_rebuild_menu (GPAOptionMenu *om)
 {
-	GtkWidget *menu_shell, *item;
-	gint pos, sel;
+	GtkWidget *menu, *item;
+	GPANode *option, *child;
+	gchar *def;
+	gint pos = 0;
+	gint sel = -1;
 
-	pos = 0;
-	sel = 0;
+	menu = gtk_menu_new ();
+	gtk_widget_show (menu);
 
-	/* Remove our previous links */
-	gtk_option_menu_remove_menu (GTK_OPTION_MENU (om->menu));
-	if (om->handler) {
-		g_signal_handler_disconnect (om->node, om->handler);
-		om->handler = 0;
+	option = GPA_KEY (om->node)->option;
+	def = gpa_node_get_value (om->node);
+
+	child = gpa_node_get_child (option, NULL);
+	while (child) {
+		const gchar *id;
+		guchar *name;
+
+		name = gpa_node_get_value (child);
+		item = gtk_menu_item_new_with_label (name);
+		g_free (name);
+
+		id = gpa_node_id (child);
+		
+		g_signal_connect (G_OBJECT (item), "activate", (GCallback) gpa_option_menu_item_activate_cb, om);
+		g_object_set_data_full (G_OBJECT (item), "node", child, (GtkDestroyNotify) gpa_node_unref);
+		gtk_widget_show (item);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+		if (GPA_NODE_ID_COMPARE (child, def))
+		    sel = pos;
+
+		pos++;
+		child = gpa_node_get_child (option, child);
 	}
-
-	menu_shell = gtk_menu_new ();
-	gtk_widget_show (menu_shell);
-
-	om->node = gpa_node_get_child_from_path (GNOME_PRINT_CONFIG_NODE (GPA_WIDGET (om)->config), om->key);
-	
-	if (om->node) {
-		GPANode *option;
-		guchar *current;
-		om->handler = g_signal_connect (G_OBJECT (om->node), "modified",
-						(GCallback) gpa_option_menu_node_modified_cb, om);
-		current = gpa_node_get_value (om->node);
-		option = gpa_node_get_child_from_path (om->node, "Option");
-		if (option) {
-			GPANode *item;
-			for (item = gpa_node_get_child (option, NULL); item != NULL; item = gpa_node_get_child (option, item)) {
-				guchar *id, *name;
-				id = gpa_node_get_value (item);
-				name = gpa_node_get_path_value (item, "Name");
-				if (id && *id && name && *name) {
-					GtkWidget *i;
-					gpa_node_ref (item);
-					i = gtk_menu_item_new_with_label (name);
-					g_object_set_data_full (G_OBJECT (i), "node", item,
-								(GtkDestroyNotify) gpa_node_unref);
-					g_signal_connect (G_OBJECT (i), "activate",
-							  (GCallback) gpa_option_menu_item_activate_cb, om);
-					gtk_widget_show (i);
-					gtk_menu_shell_append (GTK_MENU_SHELL (menu_shell), i);
-					if (current && !strcmp (id, current))
-						sel = pos;
-					pos += 1;
-				}
-				if (name)
-					g_free (name);
-				if (id)
-					g_free (id);
-				gpa_node_unref (item);
-			}
-			gpa_node_unref (option);
-		}
-		if (current)
-			g_free (current);
-	}
-
 	if (pos < 1) {
 		item = gtk_menu_item_new_with_label (_("No options are defined"));
 		gtk_widget_set_sensitive (item, FALSE);
 		gtk_widget_show (item);
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu_shell), item);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	}
+	if (sel == -1) {
+		g_warning ("rebuild_menu_cb, could not set value of %s to %s",
+			   gpa_node_id (option), def);
+		sel = 0;
 	}
 
-	gtk_widget_show (menu_shell);
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (om->menu), menu_shell);
+	gtk_widget_show (menu);
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (om->menu), menu);
 	gtk_option_menu_set_history (GTK_OPTION_MENU (om->menu), sel);
+	g_free (def);
+}
+
+/**
+ * gpa_option_menu_node_modified_cb:
+ * @node: 
+ * @flags: 
+ * @om: 
+ * 
+ * Callback for when the value of the node we are a view/control
+ * for changes.
+ **/
+static void
+gpa_option_menu_node_modified_cb (GPANode *node, guint flags, GPAOptionMenu *om)
+{
+	if (om->updating)
+		return;
+
+	gpa_option_menu_rebuild_menu (om);
 }
 
 static void
-gpa_option_menu_parent_modified_cb (GPANode *node, guint flags, GPAOptionMenu *om)
+gpa_option_menu_connect (GPAOptionMenu *om)
 {
-	if (om->updating) {
-		g_print ("Avoid updating parent_modified_cb\n");
+	GPANode *node;
+
+	g_assert (om->node == NULL);
+	g_assert (om->handler == 0);
+	g_assert (om->updating == FALSE);
+	
+	node = gpa_node_lookup (om->config, om->key);
+	if (!node) {
+		GtkWidget *menu, *item;
+		gtk_option_menu_remove_menu (GTK_OPTION_MENU (om->menu));
+		menu = gtk_menu_new ();
+		gtk_widget_show (menu);
+		item = gtk_menu_item_new_with_label (_("No options are defined"));
+		gtk_widget_set_sensitive (item, FALSE);
+		gtk_widget_show (item);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+		gtk_option_menu_set_menu (GTK_OPTION_MENU (om->menu), menu);
 		return;
 	}
-
-	om->updating = TRUE;
+	
+	om->node = node;
+	om->handler = g_signal_connect (G_OBJECT (node), "modified",
+					(GCallback) gpa_option_menu_node_modified_cb, om);
 	gpa_option_menu_rebuild_menu (om);
-	om->updating = FALSE;
 }
 
-static gboolean
-gpa_option_menu_construct (GPAOptionMenu *om, const guchar *key, GPANode *parent)
+/**
+ * gpa_option_menu_config_modified_cb:
+ * @node: 
+ * @flags: 
+ * @om: 
+ * 
+ * Callback for "modified" signal of GPAConfig, called when either the active printer
+ * or settings has changed.
+ * 
+ **/
+static void
+gpa_option_menu_config_modified_cb (GPANode *node, guint flags, GPAOptionMenu *om)
 {
-	g_return_val_if_fail (GPA_IS_NODE (parent), FALSE);
-
-	om->key = g_strdup (key);
-
-	gpa_option_menu_rebuild_menu (om);
-
-	om->parent = gpa_node_ref (parent);
-	om->parent_handler = g_signal_connect (G_OBJECT (parent), "modified",
-					       (GCallback) gpa_option_menu_parent_modified_cb, om);
-
-	return TRUE;
+	gpa_option_menu_disconnect (om);
+	gpa_option_menu_connect (om);
 }
 
+/**
+ * gpa_option_menu_new:
+ * @gpc: 
+ * @key: 
+ * 
+ * Craete a new Option menu that selects the node pointed by @key.
+ * 
+ * Return Value: A new widget, NULL on error
+ **/
 GtkWidget *
-gpa_option_menu_new (GnomePrintConfig *config, const guchar *key, GPANode *parent)
+gpa_option_menu_new (GnomePrintConfig *gpc, const guchar *key)
 {
-	GtkWidget *gpa_menu;
-
-	g_return_val_if_fail (config != NULL, NULL);
+	GPAOptionMenu *om;
+	
+	g_return_val_if_fail (gpc != NULL, NULL);
 	g_return_val_if_fail (key != NULL, NULL);
-	g_return_val_if_fail (parent != NULL, NULL);
 
-	gpa_menu = gpa_widget_new (GPA_TYPE_OPTION_MENU, config);
+	om = (GPAOptionMenu *) gpa_widget_new (GPA_TYPE_OPTION_MENU, gpc);
+	om->key    = g_strdup (key);
+	om->config = GNOME_PRINT_CONFIG_NODE (gpc);
+	om->handler_config = g_signal_connect (G_OBJECT (om->config), "modified",
+					       (GCallback) gpa_option_menu_config_modified_cb, om);
+	/* FIXME: do we ref om->config? what happens if it dies? (Chema) */
+	gpa_option_menu_connect (om);
 
-	gpa_option_menu_construct (GPA_OPTION_MENU (gpa_menu), key, parent);
-
-	gtk_widget_show (gpa_menu);
-
-	return gpa_menu;
+	return (GtkWidget *)om;
 }
 
-
