@@ -40,6 +40,7 @@
 #include <libgnomecanvas/gnome-canvas-util.h>
 #include <libgnomeprint/private/gpa-node.h>
 #include <libgnomeprint/private/gnome-print-private.h>
+#include <libgnomeprint/private/gnome-print-config-private.h>
 
 #include "gpa-paper-preview-item.h"
 
@@ -108,9 +109,11 @@ struct _GpaPaperPreviewItem {
 	guint32 color_lmargin_hl;
 	guint32 color_stripe;
 
-	/* Needed so that we can disconect after use */
-	GPANode *node_media, *node_printer, *node_page;
-	guint handler_media, handler_printer, handler_page;
+	/* Nodes and handlers we are listening to */
+	GPANode *gpa_config;
+	GPANode *node[4];
+	gulong handler[4];
+	gulong handler_config;
 };
 
 struct _GpaPaperPreviewItemClass {
@@ -208,21 +211,11 @@ gpa_paper_preview_item_finalize (GObject *object)
 		pp->right = NULL;
 	}
 
-	if (pp->node_printer) {
-		g_signal_handler_disconnect (pp->node_printer, pp->handler_printer);
-		pp->handler_printer = 0;
-		pp->node_printer = gpa_node_unref (pp->node_printer);
-	}
-	if (pp->node_media) {
-		g_signal_handler_disconnect (pp->node_media, pp->handler_media);
-		pp->handler_media = 0;
-		pp->node_media = gpa_node_unref (pp->node_media);
-	}
-	if (pp->node_page) {
-		g_signal_handler_disconnect (pp->node_page, pp->handler_page);
-		pp->handler_page = 0;
-		pp->node_page = gpa_node_unref (pp->node_page);
-	}
+	gpa_paper_preview_item_disconnect (pp);
+
+	g_signal_handler_disconnect (G_OBJECT (pp->gpa_config), pp->handler_config);
+	pp->handler_config = 0;
+	pp->gpa_config = NULL;
 
 	G_OBJECT_CLASS (item_parent_class)->finalize (object);
 }
@@ -787,12 +780,59 @@ gpa_paper_preview_item_style_color_cb (GtkWidget *widget, GtkStyle *ps, GpaPaper
 	gpa_paper_preview_item_load_colors (item, widget);
 }
 
+static void
+gpa_paper_preview_item_disconnect (GpaPaperPreviewItem *pp)
+{
+	gint max, i;
+
+	max = sizeof (pp->node) / sizeof (GPANode *);
+	for (i = 0; i < max; i++) {
+		if (pp->handler[i]) {
+			g_signal_handler_disconnect (pp->node[i],
+						     pp->handler[i]);
+			pp->handler[i] = 0;
+			gpa_node_unref (pp->node[i]);
+			pp->node[i] = NULL;
+		}
+	}
+}
+				   
+static void
+gpa_paper_preview_item_connect (GpaPaperPreviewItem *pp)
+{
+	const guchar *paths[] = {
+		GNOME_PRINT_KEY_PAPER_SIZE,
+		GNOME_PRINT_KEY_PAPER_ORIENTATION,
+		GNOME_PRINT_KEY_PAGE_ORIENTATION,
+		GNOME_PRINT_KEY_LAYOUT,
+	};
+	gint i, max;
+	
+	max = sizeof (pp->node) / sizeof (GPANode *);
+	g_assert (max == (sizeof (paths) / sizeof (guchar *)));
+	for (i = 0; i < max; i++) {
+		pp->node[i] = gpa_node_get_child_from_path (pp->gpa_config, paths[i]);
+		if (pp->node[i])
+			pp->handler[i] = g_signal_connect (G_OBJECT (pp->node[i]), "modified",
+							   (GCallback) gpa_paper_preview_item_reload_values, pp);
+		else
+			pp->handler[i] = 0;
+	}
+}
+
+static void
+gpa_paper_preview_item_config_modified_cb (GPANode *node, guint flags, GpaPaperPreviewItem *pp)
+{
+	gpa_paper_preview_item_disconnect (pp);
+	gpa_paper_preview_item_connect (pp);
+	gpa_paper_preview_item_reload_values (NULL, 0, pp);
+}
+
 GnomeCanvasItem *
 gpa_paper_preview_item_new (GnomePrintConfig *config, GtkWidget *canvas)
 {
 	GpaPaperPreviewItem *pp;
 	GnomeCanvasItem *item;
-	GPANode *node = NULL;
 
 	g_return_val_if_fail (config != NULL, NULL);
 	g_return_val_if_fail (canvas != NULL, NULL);
@@ -802,30 +842,15 @@ gpa_paper_preview_item_new (GnomePrintConfig *config, GtkWidget *canvas)
 	pp = GPA_PAPER_PREVIEW_ITEM (item);
 
 	pp->config = gnome_print_config_ref (config);
-
+	pp->gpa_config = GNOME_PRINT_CONFIG_NODE (pp->config);
+	pp->handler_config = g_signal_connect (G_OBJECT (pp->gpa_config), "modified",(GCallback)
+					       gpa_paper_preview_item_config_modified_cb, pp);
 	gpa_paper_preview_item_load_colors (pp, canvas);
 	g_signal_connect (G_OBJECT (canvas), "style_set", G_CALLBACK (gpa_paper_preview_item_style_color_cb), pp);
-	gpa_paper_preview_item_reload_values (node, 0, pp);
-	
-	node = gpa_node_get_child_from_path (GNOME_PRINT_CONFIG_NODE (config), "Printer");
-	if (node) {
-		pp->node_printer = node;
-		pp->handler_printer = g_signal_connect (G_OBJECT (node), "modified",
-							(GCallback) gpa_paper_preview_item_reload_values, pp);
-	}
-	node = gpa_node_get_child_from_path (GNOME_PRINT_CONFIG_NODE (config), "Settings.Output.Media");
-	if (node) {
-		pp->node_media = node;
-		pp->handler_media = g_signal_connect (G_OBJECT (node), "modified",
-						      (GCallback) gpa_paper_preview_item_reload_values, pp);
-	}
-	node = gpa_node_get_child_from_path (GNOME_PRINT_CONFIG_NODE (config), "Settings.Document.Page");
-	if (node) {
-		pp->node_page = node;
-		pp->handler_page = g_signal_connect (G_OBJECT (node), "modified",
-						     (GCallback) gpa_paper_preview_item_reload_values, pp);
-	}
+	gpa_paper_preview_item_reload_values (NULL, 0, pp);
 
+	gpa_paper_preview_item_connect (pp);
+	
 	return item;
 }
 
